#ifndef __Article_h__
#define __Article_h__

#include <ctype.h>
#include <stdio.h>

#include <string>

#include "NSError.h"
#include "Debug.h"
#include "readline.h"

/**
 * \class InvalidArticleError
 * \author Thomas Gschwind
 * Syntactic error in Article dectected
 */
class InvalidArticleError:public NSError {
      public:
	InvalidArticleError(const char *txt, const char *file,
			    const char *function, int line):NSError(txt,
								    file,
								    function,
								    line) {
	} InvalidArticleError(const string & txt, const char *file,
			      const char *function, int line):NSError(txt,
								      file,
								      function,
								      line)
	{
	}

	virtual void print() {
		slog << "Exception!\n"
		    << "  Type: InvalidArticle\n"
		    << "  Desc: " << _errtext << "\n";
	}
};

/**
 * \class Article
 * \author Thomas Gschwind
 *
 * Handle a Article
 * \bug Method documentation is missing.
 */
class Article {
      private:

	/**
	 * Return the position of a given field in the article's header.
	 * \param ifld The name of the field to be searched for including `:'
	 * \throw InvalidArticleError If a syntactic error is detected.
	 */
	const char *find_field(const char *ifld) const;

      protected:
	int _nbr;
	string _text;
	const char *_ctext;

      public:
	enum {
		Head = 0x1,
		Body = 0x2
	};

	Article() {
	}
	Article(Article * a) {
		_nbr = a->getnbr();
		_text = a->gettext();
		_ctext = _text.c_str();
	}
	Article(int artnbr) {
		setnbr(artnbr);
	}
	Article(int artnbr, const char *text, int textlen = 0) {
		setnbr(artnbr);
		_text.assign(text, textlen);
		_ctext = _text.c_str();
	}
	~Article() {
	}

	void read(std::istream & is) {
		DEBUG(slog.p(Logger::Debug) << "Article::read(&is)\n");
		_text = "";
		readtext(is, _text);
		_ctext = _text.c_str();
	}

	void setnbr(int nbr) {
		_nbr = nbr;
	}
	void clear() {
		_nbr = 0;
		_text = "";
		_ctext = _text.c_str();
	}
	void settext(const string & text) {
		_text = text;
		_ctext = _text.c_str();
	}

	int getnbr() const {
		return _nbr;
	} string gettext() {
		return _text;
	}
	const char *c_str() {
		return _ctext;
	}
	int length() const {
		return _text.length();
	} int has_field(const char *ifld) {
		char c;

		c = *find_field(ifld);
		if (!c || c == '\r' || c == '\n')
			return 0;
		return 1;
	}

	/*
	 * Return a header field of the article.
	 * \param ifld The name of the field to be extracted including `:'
	 * \throw NoSuchFieldError raised, if fld is not part of the
	 * 	overview database
	 */
	string getfield(const char *ifld, int Full = 0) const;

	void setfield(const char *ifld, const char *field_data);

	std::ostream & write(std::ostream & os, int flags = Head | Body);

	friend std::ostream & operator<<(std::ostream & os, Article & art) {
		os << art._text;
		return os;
	}
};

#endif
